function [total_cost,Bits,model_cost]= DFT_MDL(data,num_coeff)

% data = randn(1,1000);
data=zscore(data);    % z normalization 

Int_data = DNorm_Unif(data,256);          % data in discrete space

Bits = zeros(1,length(num_coeff));                % Bits is the array to store bit cost for each cardinality

model_cost = zeros(1,length(num_coeff));

for i=1:length(num_coeff);
   
    
    line_dft = ts2dft(Int_data,num_coeff(i));
    Bits(i) =calculate_bit_cost(Int_data,line_dft);    % the dimensionality * bit cost   
     model_cost(i)= num_coeff(i)*log2(length(data)/2)+ num_coeff(i)*16;
    
end
  total_cost=Bits+model_cost;
  
  

end


function DZ = DNorm_Unif(X, card)

% Input: X ts is a subsequence, a row vector
% Output: DZ is a discrete subsequence in range [1,card]
    if (~exist('card','var')), card=256; end;
      
    mn = min(X);
    mx = max(X);
    DZ = round((X-mn)/(mx-mn)*(card-1)) + 1;
    DZ=DZ-card/2;
end

function [bits]=calculate_bit_cost(Int_data,Int_line_position)
% Input: the integer data, Dimensionality, alphabet size, Intger
%        reconstruction line poine
% Output: 
%    bits :the bit cost of this time series


Int_dist=Int_data-Int_line_position;                              % calculate the distance between Int data and Int reconstruction line;

uniques=unique(Int_dist);             % unique numbers in Int_dist;
numUnique=histc(Int_dist,uniques);    %  how many times each unique number appears

Prob= numUnique/length(Int_dist);

if uniques == 0
    avg = 1;
    total_bits = avg*length(Int_data);
    num_of_uniques=length(uniques);
    largest_unique=max(abs(uniques));
    
else
    
[dict,avg]=huffmandict(uniques,Prob);
bits=avg*length(Int_data);
end

end

function NewDFT = ts2dft( data,coeff)


N = length(data);
fa_N = fft(data); % copy fourier
fb_N = fa_N(1:floor(end/2)+1);
fc_N = (real(fb_N).^2 + imag(fb_N).^2);
[s ix] = sort(fc_N,'descend');

fd_N = zeros(size(fa_N));
fd_N(ix(1:coeff)) = fa_N(ix(1:coeff));
%fd_N(N+1-ix(1:coeff)) = fa_N(N+1-ix(1:coeff));
for i=1:coeff
   if (ix(i)~=1) && (ix(i)~=(N/2)+1)
      fd_N(N+2-ix(i)) = fa_N(N+2-ix(i)); 
   end
end

 NewDFT = round(real(ifft(fd_N))); % reconstruction

figure(1); 
plot(data,'r');
hold on;
plot(NewDFT); 
hold off;
title(sprintf('1 Sin Wave when coeff=%d',coeff));

%fprintf('coef=%d   min=%f max=%f\n',coeff,min(fa_N),max(fa_N));
fprintf('coef=%d\n',coeff);
axis([-inf inf -127 128]);
   
end








