
function [total_cost,Bits,model_cost] = PLA_MDL(data,num_seg)


data=zscore(data);    % z normalization 

Int_data = DNorm_Unif(data,256);          % data in discrete space

uniques_num = length(unique(Int_data));
Bits = zeros(1,length(num_seg));                % Bits is the array to store bit cost for each cardinality
num_of_unique_numbers=zeros(1,length(num_seg)); % num_of_unique_numbers is how many  unique numbers given to huffman coding for each cardinality
largest_dist=zeros(1,length(num_seg));          % largest unique number given to huffman coding for each cardinality
model_cost = zeros(1,length(num_seg));

for i=1:length(num_seg);
    tic;
    line_pla = ts2pla(Int_data,num_seg(i));
    [Bits(i),num_of_unique_numbers(i),largest_dist(i)] =calculate_bit_cost(Int_data,line_pla);    % the dimensionality * bit cost   
     model_cost(i)=2*log2(256)*num_seg(i) + log2(length(data))*num_seg(i);
     toc;
end

 total_cost=Bits+model_cost;


end

function [bits]=calculate_bit_cost(Int_data,Int_line_position)
% Input: the integer data, Dimensionality, alphabet size, Intger
%        reconstruction line poine
% Output: 
%    bits :the bit cost of this time series


Int_dist=Int_data-Int_line_position;                              % calculate the distance between Int data and Int reconstruction line;

uniques=unique(Int_dist);             % unique numbers in Int_dist;
numUnique=histc(Int_dist,uniques);    %  how many times each unique number appears

Prob= numUnique/length(Int_dist);

if uniques == 0
    avg = 1;
    total_bits = avg*length(Int_data);
    num_of_uniques=length(uniques);
    largest_unique=max(abs(uniques));
    
else
    
[dict,avg]=huffmandict(uniques,Prob);
bits=avg*length(Int_data);
end

end

function segment = bottom_up(data,num_segments,forceplot)
% This function approximates a time series by a sequence of piecewise linear segments. 
% This version is uncommented, email the author for details.

data = data';
if num_segments == 1;
    
   coef = polyfit([1:length(data)]',data,1);
   y = polyval(coef,1:length(data));
   segment.lx = 1;
   segment.rx = length(data);
   segment.ly = y(1);
   segment.ry = y(2);
else

left_x = [1 : 2 : length(data)-1];
right_x      = left_x + 1;
right_x(end) = length(data);
number_of_segments = length(left_x );

    
    
for i = 1 : number_of_segments 
   
   segment(i).lx = left_x(i);
   segment(i).rx = right_x(i);
   segment(i).mc = inf;
end;

for i = 1 : number_of_segments - 1
   coef = polyfit([segment(i).lx :segment(i+1).rx]',data(segment(i).lx :segment(i+1).rx),1);
   best = (coef(1)*( [segment(i).lx :segment(i+1).rx]' ))+coef(2);
   segment(i).mc = sum((data([segment(i).lx :segment(i+1).rx]')-best).^2);
end;

while  length(segment) > num_segments  
   
   
   [value, i ] = min([segment(:).mc]);
      
  
   if i > 1 & i < length(segment) -1								
      
    	coef = polyfit([segment(i).lx :segment(i+2).rx]',data(segment(i).lx :segment(i+2).rx),1);
    	best = (coef(1)*( [segment(i).lx :segment(i+2).rx]' ))+coef(2);
       
        segment(i).mc = sum((data([segment(i).lx :segment(i+2).rx]')-best).^2);

	 	segment(i).rx = segment(i+1).rx;
    	segment(i+1) = [];
    
    	i = i - 1;
    
        coef = polyfit([segment(i).lx :segment(i+1).rx]',data(segment(i).lx :segment(i+1).rx),1);
    	best = (coef(1)*( [segment(i).lx :segment(i+1).rx]' ))+coef(2);
    
      segment(i).mc = sum((data([segment(i).lx :segment(i+1).rx]')-best).^2);
       
   elseif i == 1
       
	    coef = polyfit([segment(i).lx :segment(i+2).rx]',data(segment(i).lx :segment(i+2).rx),1);
    	best = (coef(1)*( [segment(i).lx :segment(i+2).rx]' ))+coef(2);
    	segment(i).mc = sum((data([segment(i).lx :segment(i+2).rx]')-best).^2);

	 	segment(i).rx = segment(i+1).rx;
        segment(i+1) = [];
              
   else
     
      segment(i).rx = segment(i+1).rx;
      segment(i).mc = inf;
      segment(i+1) = [];
    
      i = i - 1;
       
      coef = polyfit([segment(i).lx :segment(i+1).rx]',data(segment(i).lx :segment(i+1).rx),1);
      best = (coef(1)*( [segment(i).lx :segment(i+1).rx]' ))+coef(2);
    
      segment(i).mc = sum((data([segment(i).lx :segment(i+1).rx]')-best).^2);

   end;
           
end;




%----------------------------------------------

residuals=[];

for i = 1 : length(segment) 
   
        coef = polyfit([segment(i).lx :segment(i).rx]',data(segment(i).lx :segment(i).rx),1);
    	best = (coef(1)*( [segment(i).lx :segment(i).rx]' ))+coef(2);
        residuals = [    residuals ; sum((data([segment(i).lx :segment(i).rx]')-best).^2)];
end;




if nargin > 2

    hold on;
    plot(data,'r');
    
end; 


    
    temp = [];
    for i = 1 : length(segment) 
   
      coef = polyfit([segment(i).lx :segment(i).rx]',data(segment(i).lx :segment(i).rx),1);
      best = (coef(1)*( [segment(i).lx :segment(i).rx]' ))+coef(2);
      
      segment(i).ly = best(1);
      segment(i).ry = best(end);
      segment(i).mc = [];
      
        if nargin > 2
            plot([segment(i).lx :segment(i).rx]', best,'b');
        end;
  
      temp = [temp; [segment(i).ly segment(i).ry]];
   
    end;

    
if nargin > 2
    for i = 1 : length(segment)  - 1 
       % plot([segment(i).rx :segment(i+1).lx]', [ temp(i,2) temp(i+1,1)  ],'g');
    end;
end;
end
end

% Input: X ts is a subsequence, a row vector
% Output: DZ is a discrete subsequence in range [1,card]
function DZ = DNorm_Unif(X, card)
    if (~exist('card','var')), card=256; end;
      
    mn = min(X);
    mx = max(X);
    DZ = round((X-mn)/(mx-mn)*(card-1)) + 1;
    DZ=DZ-card/2;
    %DZ = zscore(DZ);
end
% Convert segment from PLA to discrete time series
% ======================================
% == Input ==  a time series in a row vector; 
%              number of segments;
%===============================================
% segment : an output from PLA()
%       - segment(i).lx  -> left position on x axis
%       - segment(i).rx  -> right position on x axis
%       - segment(i).mc  -> error (don't sure)
%       - segment(i).y   -> y position of this interval
% == Output ==
% TS     : a discrete ts which has the same length as input ts (in high dimensional space) before using apca()
% ======================================
% last updated: 29 Nov 2010
function TS = ts2pla(data,num_segment)
   segment=bottom_up(data,num_segment);
   TS(segment(end).rx)=0;
    for i=1:length(segment)
        x = [segment(i).lx, segment(i).rx];
        Y = [segment(i).ly, segment(i).ry];
        xi = segment(i).lx:segment(i).rx;
       TS(segment(i).lx:segment(i).rx) = round(interp1(x,Y,xi));
    end
figure(1);
plot(data);
hold on;
plot(TS,'r'); 
hold off;
title(sprintf('PLA number of segment =%d ', num_segment));
axis([-inf inf -127 128]);

end









